@@ -13,7 +13,7 @@ require Exporter;
 @EXPORT = qw(
 	
 );
-$VERSION = '2.07_05';
+$VERSION = '2.05';
 
 
 # Preloaded methods go here.
@@ -27,7 +27,6 @@ use Tangram::Array;
 use Tangram::IntrArray;
 
 use Tangram::Hash;
-use Tangram::IntrHash;
 
 sub connect
   {
