@@ -115,98 +115,42 @@ sub includes
 	my ($self, $item) = @_;
 	my ($coll, $memdef) = @$self;
 
-	my $schema = $coll->{storage}{schema};
-
 	my $coll_tid = $coll->root_table;
 
 	my $link_tid = Tangram::Alias->new;
 	my $coll_col = $memdef->{coll};
 	my $item_col = $memdef->{item};
 
-	my $objects = Set::Object->new
-	    (
-	     $coll,
-	     Tangram::LinkTable->new($memdef->{table}, $link_tid)
-	    );
-	my $target;
+	my $objects = Set::Object->new($coll, Tangram::LinkTable->new($memdef->{table}, $link_tid) ),
+		my $target;
 
-	if (ref $item) {
-	    if ($item->isa('Tangram::QueryObject'))
+	if (ref $item)
+	{
+		if ($item->isa('Tangram::QueryObject'))
 		{
-		    $target = 't' . $item->object->root_table . '.' . $schema->{sql}{id_col};
-		    $objects->insert( $item->object );
+			$target = 't' . $item->object->root_table . '.id';
+			$objects->insert( $item->object );
 		}
-	    else
+		else
 		{
-		    $target = $coll->{storage}->export_object($item)
-			or die "'$item' is not a persistent object";
+			$target = $coll->{storage}->export_object($item)
+				or die "'$item' is not a persistent object";
 		}
 	}
 	else
-	    {
+	{
 		$target = $item;
-	    }
+	}
 
 	Tangram::Filter->new
-		(
-		 expr => "t$link_tid.$coll_col = t$coll_tid.$schema->{sql}{id_col} AND t$link_tid.$item_col = $target",
-		 tight => 100,      
-		 objects => $objects,
-		 link_tid => $link_tid # for Sequence prefetch
-		);
+			(
+			 expr => "t$link_tid.$coll_col = t$coll_tid.id AND t$link_tid.$item_col = $target",
+			 tight => 100,      
+			 objects => $objects,
+			 link_tid => $link_tid # for Sequence prefetch
+			);
 }
 
-sub includes_or {
-    my ($self, @items) = @_;
-    my ($coll, $memdef) = @$self;
-
-    my $schema = $coll->{storage}{schema};
-    my $coll_tid = $coll->root_table;
-
-    my $link_tid = Tangram::Alias->new;
-    my $coll_col = $memdef->{coll};
-    my $item_col = $memdef->{item};
-
-    my $objects = Set::Object->new
-	($coll,
-	 Tangram::LinkTable->new($memdef->{table}, $link_tid)
-	);
-    my @targets;
-
-    foreach my $item (@items) {
-        if (ref $item) {
-            if ($item->isa('Tangram::QueryObject'))
-              {
-                  push @targets, ('t' . $item->object->root_table.'.'
-				  . $schema->{sql}{id_col});
-                  $objects->insert( $item->object );
-              }
-            else
-              {
-                  push @targets, ($coll->{storage}->export_object($item)
-                                  or die "'$item' is not a persistent
-object"
-                                 );
-              }
-        }
-        else {
-            push @targets, $item;
-        }
-    }
-
-    my $joined_targets = join(',', @targets);
-    
-        Tangram::Filter->new
-        (
-         expr => "t$link_tid.$coll_col = t$coll_tid.$schema->{sql}{id_col} AND
-t$link_tid.$item_col IN ($joined_targets)",
-         tight => 100,      
-         objects => $objects,
-         link_tid => $link_tid # for Sequence prefetch
-        );
-}
-
-
 use overload '<' => \&includes;
 
 package Tangram::IntrCollExpr;
@@ -221,7 +165,6 @@ sub includes
 	my $coll_tid = $coll->root_table;
 	my $item_class = $memdef->{class};
 	my $storage = $coll->{storage};
-	my $schema = $storage->{schema};
 
 	my $item_id;
 
@@ -233,7 +176,7 @@ sub includes
 
 			return Tangram::Filter->new
 				(
-				 expr => "t$item_tid.$memdef->{coll} = t$coll_tid.$schema->{sql}{id_col}",
+				 expr => "t$item_tid.$memdef->{coll} = t$coll_tid.id",
 				 tight => 100,
 				 objects => Set::Object->new($coll, $item->object),
 				)
@@ -248,81 +191,7 @@ sub includes
 	}
 
 	my $remote = $storage->remote($item_class);
-	# FIXME - style inconsistency
-	return ($self->includes($remote) & ($remote->{id} == $item_id));
-}
-
-sub includes_or
-{
-	my ($self, @items) = @_;
-	my ($coll, $memdef) = @$self;
-	my $coll_tid = $coll->root_table;
-	my $item_class = $memdef->{class};
-	my $item_tid;
-	my $storage = $coll->{storage};
-	my $schema = $storage->{schema};
-
-	my (@targets_fwd, @targets_rev);
-	my $objects = Set::Object->new
-	    ($coll,
-	    );
-
-	foreach my $item (@items) {
-	    if (ref($item))
-		{
-		    if ($item->isa('Tangram::QueryObject'))
-			{
-			    $item_tid = $item->object->table($item_class);
-			    push @targets_fwd, ("t".$item_tid.".$memdef->{coll}");
-			    $objects->insert($item->object);
-			}
-		    else
-			{
-			    # 
-			    #push @targets, ($storage->export_object($item));
-			    push @targets_rev, ($storage->export_object($item));
-			}
-		}
-	    else
-		{
-		    push @targets_rev, $storage->{export_id}->($item);
-		}
-	}
-
-	my $expr;
-	if (@targets_fwd) {
-	    my  $joined_targets = join(',', @targets_fwd);
-	    $expr =
-	    Tangram::Filter->new
-		    (
-		     expr => "(t$coll_tid.$schema->{sql}{id_col} IN ($joined_targets))",
-		     tight => 120,
-		     objects => $objects,
-		    );
-	}
-	if (@targets_rev) {
-
-	    my $remote = $storage->remote($item_class);
-	    #$objects->insert($remote);
-	    my $item_tid = $remote->object->table($item_class);
-
-	    my $joined_targets = join(',', @targets_rev);
-	    my $new_expr = 
-		Tangram::Filter->new
-			(
-			 expr => "(t$item_tid.$schema->{sql}{id_col} in ($joined_targets))",
-			 tight => 100,
-			 objects => $objects,
-			);
-
-	    if ($expr) {
-		return ( ( $self->includes($remote) & $new_expr ) | $expr );
-	    }
-
-	    return ( $self->includes($remote) & $new_expr );
-	}
-	return $expr;
-
+	return $self->includes($remote) & $remote->{id} == $item_id;
 }
 
 package Tangram::LinkTable;
@@ -348,8 +217,6 @@ sub where
 
 package Tangram::CollOnDemand;
 
-use Carp qw(confess);
-
 sub TIESCALAR
 {
 	my $pkg = shift;
@@ -360,21 +227,18 @@ sub FETCH
 {
 	my $self = shift;
 	my ($type, $def, $storage, $id, $member, $class) = @$self;
-	my $obj = $storage->{objects}{$id}
-	    or confess "FETCH failed to get object $id!";
+	my $obj = $storage->{objects}{$id} or die;
 	my $coll = $type->demand($def, $storage, $obj, $member, $class);
 	untie $obj->{$member};
 	$obj->{$member} = $coll;
-	my ($pkg,$fn,$l) = caller;
-	return $coll;
 }
 
 sub STORE
 {
 	my ($self, $coll) = @_;
 	my ($type, $def, $storage, $id, $member, $class) = @$self;
-	my $obj = $storage->{objects}{$id}
-	    or confess "FETCH failed to get object $id!";
+
+	my $obj = $storage->{objects}{$id} or die;
 	$type->demand($def, $storage, $obj, $member, $class);
 
 	untie $obj->{$member};
