@@ -5,28 +5,20 @@ use strict;
 package Tangram::Storage;
 use DBI;
 use Carp;
-use Tangram::Core qw(pretty);
 
 use vars qw( %storage_class );
 
 BEGIN {
-    eval 'use Scalar::Util qw()';
-    if ($@) {
-	*Tangram::refaddr = sub { (shift) + 0 };
-	eval 'use WeakRef';
-	if ($@) {
-	    *Tangram::weaken = sub { };
-	    $Tangram::no_weakrefs = 1;
-	} else {
-	    *Tangram::weaken = \&WeakRef::weaken;
-	    $Tangram::no_weakrefs = 0;
-	}
-    } else {
-	*Tangram::weaken = \&Scalar::Util::weaken;
-	*Tangram::refaddr = \&Scalar::Util::refaddr;
-	$Tangram::no_weakrefs = 0;
-    }
-    *pretty = *Tangram::Core::pretty;
+
+  eval { require 'WeakRef.pm' };
+
+  if ($@) {
+    *Tangram::weaken = sub { };
+    $Tangram::no_weakrefs = 1;
+  } else {
+    *Tangram::weaken = \&WeakRef::weaken;
+    $Tangram::no_weakrefs = 0;
+  }
 }
 
 sub new
@@ -57,21 +49,9 @@ sub split_id
 sub combine_ids
   {
 	my $self = shift;
-	defined(my $id = shift) or confess "no id";
-	defined(my $cid = shift) or confess "no cid";
-	defined($self->{cid_size}) or die "no CID size in schema";
-	return ( $self->{layout1}
-		 ? shift
-		 : sprintf("%d%0$self->{cid_size}d", $id, $cid) );
+	return $self->{layout1} ? shift : sprintf("%d%0$self->{cid_size}d", @_);
   }
 
-sub dbms_date
-    {
-	my $self = shift;
-	my $driver = $self->{driver} or confess "no driver";
-	return $self->{driver}->dbms_date(shift);
-    }
-
 sub _open
   {
     my ($self, $schema) = @_;
@@ -121,30 +101,18 @@ sub _open
 	{
 	  my ($obj, $id) = @_;
 
-	  if ($Tangram::TRACE && ($Tangram::DEBUG_LEVEL > 2)) {
-		if ($id) {
-		   print $Tangram::TRACE "Welcoming $obj as $id\n";
-		} else {
-		   print $Tangram::TRACE "Un-welcoming $obj\n";
-		}
-	  }
 	  if ($id) {
-	    $self->{ids}{Tangram::refaddr($obj)} = $id;
+	    $self->{ids}{0 + $obj} = $id;
 	  } else {
-	    delete $self->{ids}{Tangram::refaddr($obj)};
+	    delete $self->{ids}{0 + $obj};
 	  }
 	};
 
     $self->{get_id} = $schema->{get_id} || sub {
-	  my $obj = shift;
-	  my $address = Tangram::refaddr($obj);
+	  my $address = 0 + shift();
 	  my $id = $self->{ids}{$address};
-	  if ($Tangram::TRACE && ($Tangram::DEBUG_LEVEL > 2)) {
-		print $Tangram::TRACE "$obj is ".($id?"oid $id" : "not in storage")."\n";
-	  }
 	  return undef unless $id;
-	  # refaddr's can be re-used, but weakrefs are magic :-)
-	  return $id if defined($self->{objects}{$id});
+	  return $id if exists $self->{objects}{$id};
 	  delete $self->{ids}{$address};
 	  delete $self->{objects}{$id};
 	  return undef;
@@ -173,15 +141,7 @@ sub open_connection
     # private - open a new connection to DB for read
 
     my $self = shift;
-    my $db = DBI->connect($self->{-cs}, $self->{-user}, $self->{-pw})
-	or die;
-
-    if (exists $self->{no_tx}) {
-	$db->{AutoCommit} = ($self->{no_tx} ? 1 : 0);
-
-    }
-
-    return $db;
+    DBI->connect($self->{-cs}, $self->{-user}, $self->{-pw}) or die;
 }
 
 sub close_connection
@@ -241,9 +201,7 @@ sub prepare
   {
 	my ($self, $sql) = @_;
 	
-	print $Tangram::TRACE "Tangram::Storage: "
-	    ."preparing [@{[ $psi++ ]}] $sql\n"
-	    if $Tangram::TRACE && ($Tangram::DEBUG_LEVEL > 1);
+	print $Tangram::TRACE "preparing [@{[ $psi++ ]}] $sql\n" if $Tangram::TRACE;
 	$self->{db}->prepare($sql);
   }
 
@@ -263,8 +221,6 @@ sub make_id
 		$self->{set_mark} = 1;	# cleared by tx_start
 	  } else {
 		$id = $self->make_1st_id_in_tx();
-		$self->{mark} = $id+1;
- 		$self->{set_mark} = 1;
 	  }
 	  
 	  return sprintf "%d%0$self->{cid_size}d", $id, $class_id;
@@ -311,13 +267,7 @@ sub make_1st_id_in_tx
 	
 	$sth = $self->{make_id}{get};
 	$sth->execute();
-    my $row = $sth->fetchrow_arrayref() or
-	die "`Tangram' table corrupt; insert a valid row!";
-	my $id = $row->[0];
-    while ($row =  $sth->fetchrow_arrayref()) {
-	warn "Eep!  More than one row in `Tangram' table!";
-	$id = $row->[0] if ($row->[0] > $id);
-    }
+	my $id = $sth->fetchrow_arrayref()->[0];
 	$sth->finish();
 
 	return $id;
@@ -337,36 +287,10 @@ sub unknown_classid
     confess "class '$class' doesn't exist in this storage"
 }
 
-{
-    no strict 'refs';
 sub class_id
 {
-    my $self = shift;
-    $self->{class2id}{$_[0]} or do {
-	# crawl ISA tree...
-	my @stack = \%{$_[0]."::"};
-	my $seen = Set::Object->new(@stack);
-	while ( my $stash = pop @stack ) {
-	    my @supers = @{ *{$stash->{ISA}}{ARRAY} };
-	    for my $super ( @supers ) {
-		if ( defined $self->{class2id}{$super} ) {
-		    $self->{class2id}{$_[0]}
-			= $self->{class2id}{$super};
-		    $self->{schema}{classdef}{$_[0]}
-			= $self->{schema}{classdef}{$super};
-		    goto OK
-		}
-		else {
-		    $super = \%{$super."::"};
-		}
-	    }
-	    push @stack, grep { $seen->insert($_) } @supers;
-	}
-    OK:
-	$self->{class2id}{$_[0]};
-    } or
-	unknown_classid $_[0];
-}
+    my ($self, $class) = @_;
+    $self->{class2id}{$class} or unknown_classid $class;
 }
 
 #############################################################################
@@ -381,17 +305,6 @@ sub tx_start
 	unless (@{ $self->{tx} }) {
 	  delete $self->{set_mark};
 	  delete $self->{mark};
-	  print $Tangram::TRACE "START TRANSACTION;\n"
-	      if $Tangram::TRACE && ($Tangram::DEBUG_LEVEL > 0);
-	  unless ($self->{no_tx}) {
-	      $self->{db}->{AutoCommit} = 1;
-	      $self->{db}->{AutoCommit} = 0;
-	      #eval { $self->{db}->rollback(); };
-	      #$self->{db}->begin_work();
-	  }
-	} else {
-	  print $Tangram::TRACE "START TRANSACTION; (virtual)\n"
-	      if $Tangram::TRACE && ($Tangram::DEBUG_LEVEL > 1);
 	}
 
     push @{ $self->{tx} }, [];
@@ -432,12 +345,7 @@ sub tx_commit
     
     unless ($self->{no_tx} || @{ $self->{tx} } > 1) {
 	  # committing outer tx: commit to db
-	  print $Tangram::TRACE "COMMIT;\n"
-	      if $Tangram::TRACE && ($Tangram::DEBUG_LEVEL > 0);
 	  $self->{db}->commit;
-	} else {
-	  print $Tangram::TRACE "COMMIT; (virtual)\n"
-	      if $Tangram::TRACE && ($Tangram::DEBUG_LEVEL > 1);
 	}
 	
     pop @{ $self->{tx} };		# drop rollback subs
@@ -458,13 +366,11 @@ sub tx_rollback
 		$self->{db}->rollback if @{ $self->{tx} } == 1; # don't rollback db if nested tx
 		
 		# execute rollback subs in reverse order
-
-		if (my $rb = pop @{ $self->{tx} }) {
-		    foreach my $rollback ( @$rb )
-			{
-			    $rollback->($self);
-			}
-		}
+		
+		foreach my $rollback ( @{ pop @{ $self->{tx} } } )
+		  {
+			$rollback->($self);
+		  }
 	  }
 }
 
@@ -552,9 +458,8 @@ sub _insert
     $saving->insert($obj);
 
     my $class_name = ref $obj;
-    my $classId = $self->class_id($class_name);
-
-    my $class = $self->{schema}->classdef($class_name);
+    my $classId = $self->{class2id}{$class_name} or unknown_classid $class_name;
+	my $class = $self->{schema}->classdef($class_name);
 
     my $id = $self->make_id($classId);
 
@@ -567,18 +472,8 @@ sub _insert
 	my $sths = $self->{INSERT_STHS}{$class_name} ||=
 	  [ map { $self->prepare($_) } $engine->get_insert_statements($class) ];
 
-	my $context =
-	    { storage => $self,
-	      dbh => $dbh,
-	      id => $id,
-	      SAVING => $saving };
-
-	my @state = (
-		     $self->{export_id}->($id),
-		     $classId,
-		     $class->get_exporter({layout1 => $self->{layout1} })
-		         ->($obj, $context)
-		    );
+	my $context = { storage => $self, dbh => $dbh, id => $id, SAVING => $saving };
+	my @state = ( $self->{export_id}->($id), $classId, $class->get_exporter({layout1 => $self->{layout1} })->($obj, $context) );
 
 	my @fields = $engine->get_insert_fields($class);
 
@@ -637,19 +532,14 @@ sub _update
     my $class = $self->{schema}->classdef(ref $obj);
 	my $engine = $self->{engine};
 	my $dbh = $self->{db};
-	my $context =
-	    { storage => $self,
-	      dbh => $dbh,
-	      id => $id,
-	      SAVING => $saving };
+	my $context = { storage => $self, dbh => $dbh, id => $id, SAVING => $saving };
 
 	my @state = ( $self->{export_id}->($id), substr($id, -$self->{cid_size}), $class->get_exporter({ layout1 => $self->{layout1} })->($obj, $context) );
 	my @fields = $engine->get_update_fields($class);
 
 	my $sths = $self->{UPDATE_STHS}{$class->{name}} ||=
 	  [ map {
-		print $Tangram::TRACE "Tangram::Storage: preparing $_\n"
-		    if ( $Tangram::TRACE && ( $Tangram::DEBUG_LEVEL > 1 ) );
+		print $Tangram::TRACE "preparing $_\n" if $Tangram::TRACE;
 		$self->prepare($_)
 	  } $engine->get_update_statements($class) ];
 
@@ -768,20 +658,6 @@ sub defer
     push @{$self->{defered}}, $action;
 }
 
-sub import_object
-{
-    my $self = shift;
-    my $class = shift;
-    my @oids = @_;
-
-    # convert the `exported' object IDs to real OIDs
-    my $cid = $self->class_id($class);
-
-    @oids = map { $self->combine_ids($_, $cid) } @oids;
-
-    return $self->load(@oids);
-}
-
 sub load
 {
     my $self = shift;
@@ -825,7 +701,6 @@ sub welcome
   {
     my ($self, $obj, $id) = @_;
     $self->{set_id}->($obj, $id);
-
     Tangram::weaken( $self->{objects}{$id} = $obj );
   }
 
@@ -842,7 +717,7 @@ sub shrink
     my ($self) = @_;
 
     my $objects = $self->{objects};
-    my $prefetch = $self->{PREFETCH};
+    my $prefetch = $self->{prefetch};
 
     for my $id (keys %$objects)
       {
@@ -876,9 +751,6 @@ sub _row_to_object
     my ($self, $obj, $id, $class, $row) = @_;
 	my $context = { storage => $self, id => $id, layout1 => $self->{layout1} };
 	$self->{schema}->classdef($class)->get_importer($context)->($obj, $row, $context);
-    if (my $x=$obj->can("T2_import")) {
-	$x->($obj);
-    }
 	return $obj;
 }
 
@@ -899,14 +771,7 @@ sub _fetch_object_state
 sub get_polymorphic_select
   {
 	my ($self, $class) = @_;
-	if ( $class ) {
-	    return $self->{engine}->get_polymorphic_select
-		($self->{schema}->classdef($class), $self);
-	}
-	else {
-	    return Tangram::Relational::PolySelectTemplate
-		->new([],[],[],[],{});
-	}
+	return $self->{engine}->get_polymorphic_select($self->{schema}->classdef($class), $self);
   }
 
 sub select {
@@ -1017,13 +882,7 @@ sub sum
 {
     my ($self, $expr, $filter) = @_;
 
-    my $expr_is_array = ref($expr) eq 'ARRAY';
-
-    my $objects = Set::Object->new(
-				   $expr_is_array ? 
-				   map($_->objects, @$expr) :
-				   $expr->objects,
-				  );
+    my $objects = Set::Object->new($expr->objects);
 
     my @filter_expr;
 
@@ -1033,24 +892,13 @@ sub sum
 	@filter_expr = ( "($filter->{expr})" );
     }
 
-    my $sql = "SELECT " .
-      join(', ',
-	   map("SUM($_->{expr})",
-	       $expr_is_array ? @$expr : ($expr),
-	      ),
-	  ) . " FROM " . join(', ', map { $_->from } $objects->members);
+    my $sql = "SELECT SUM($expr->{expr}) FROM " . join(', ', map { $_->from } $objects->members);
 
     $sql .= "\nWHERE " . join(' AND ', @filter_expr, map { $_->where } $objects->members);
 
     print $Tangram::TRACE "$sql\n" if $Tangram::TRACE;
 
-    my @result = $self->{db}->selectrow_array($sql);
-
-    if ( $expr_is_array ) {
-      return wantarray ? @result : \@result;
-    } else {
-      return $result[0];
-    }
+    return ($self->{db}->selectrow_array($sql))[0];
 }
 
 sub id
@@ -1078,7 +926,7 @@ sub disconnect
 	}
     }
    
-    $self->{db}->disconnect if $self->{db_owned};
+    $self->{db}->disconnect;
 
     %$self = ();
 }
@@ -1143,43 +991,24 @@ sub connect
 
 	$opts ||= {};
 
-    @$self{ -cs, -user, -pw } = ($cs, $user, $pw);
-
-    $self->{driver} = $opts->{driver} || Tangram::Relational->new;
-
-   my $db = $opts->{dbh};
-    unless ( $db ) {
-      $db = $self->open_connection;
-      $self->{db_owned} = 1;
-    }
+    my $db = $opts->{dbh} || DBI->connect($cs, $user, $pw);
  
-	if (exists $opts->{no_tx}) {
-	  $self->{no_tx} = $opts->{no_tx};
+	if ($opts->{no_tx}) {
+	  $self->{no_tx} = 1;
 	} else {
 	  eval { $db->{AutoCommit} = 0 };
 	  $self->{no_tx} = $db->{AutoCommit};
 	}
 
-    if (exists $opts->{no_subselects}) {
-	$self->{no_subselects} = $opts->{no_subselects};
-    } else {
-	local($SIG{__WARN__})=sub{};
-	eval {
-	    my $sth = $db->prepare("select * from (select 1+1) as test");
-	    $sth->execute() or die;
-	};
-	if ($@ or $DBI::errstr) {
-	    $self->{no_subselects} = 1;
-	}
-    }
-
     $self->{db} = $db;
 
+    @$self{ -cs, -user, -pw } = ($cs, $user, $pw);
+
     $self->{cid_size} = $schema->{sql}{cid_size};
 	
     $self->_open($schema);
 
-	$self->{engine} = Tangram::Relational::Engine->new($schema, layout1 => $self->{layout1}, driver => $self->{driver});
+	$self->{engine} = Tangram::Relational::Engine->new($schema, layout1 => $self->{layout1});
 
     return $self;
 }
@@ -1188,9 +1017,9 @@ sub connection { shift->{db} }
 
 sub sql_do
 {
-    my ($self, $sql, @placeholders) = @_;
-    print $Tangram::TRACE "$sql with (@placeholders)\n" if $Tangram::TRACE;
-    my $rows_affected = $self->{db}->do($sql, {}, @placeholders);
+    my ($self, $sql) = @_;
+    print $Tangram::TRACE "$sql\n" if $Tangram::TRACE;
+	my $rows_affected = $self->{db}->do($sql);
     return defined($rows_affected) ? $rows_affected
 	  : croak $DBI::errstr;
 }
@@ -1206,10 +1035,8 @@ sub sql_prepare
 {
     my ($self, $sql, $connection) = @_;
     confess unless $connection;
-    print $Tangram::TRACE "           $sql\n" if $Tangram::TRACE;
-    my $sth = $connection->prepare($sql);
-    die "prepare failed; $DBI::errstr - SQL:\n--->\n$sql\n<---\n" unless $sth;
-    return $sth;
+    print $Tangram::TRACE "$sql\n" if $Tangram::TRACE;
+    return $connection->prepare($sql) or die;
 }
 
 sub sql_cursor
@@ -1218,7 +1045,7 @@ sub sql_cursor
 
     confess unless $connection;
 
-    print $Tangram::TRACE "new cursor for $sql\n" if $Tangram::TRACE;
+    print $Tangram::TRACE "$sql\n" if $Tangram::TRACE;
 
     my $sth = $connection->prepare($sql) or die;
     $sth->execute() or confess;
@@ -1247,79 +1074,12 @@ sub unload
     }
   }
 
-sub unload_all {
-    my $self = shift;
-    my $send_method = shift;
-
-    if ( $send_method ) {
-	my $objects = $self->{objects};
-	if ($objects and ref $objects eq "HASH") {
-	    while (my $oid = each %$objects) {
-		if (defined $objects->{$oid}) {
-		    if (my $x = UNIVERSAL::can($objects->{$oid},
-					       $send_method)) {
-			$x->($objects->{$oid});
-		    }
-		    $self->goodbye($objects->{$oid}, $oid);
-		}
-	    }
-	}
-	while (my $oid = each %$objects) {
-	    next unless defined $objects->{$oid};
-	    warn __PACKAGE__."::unload_all: cached ref to oid $oid "
-		."is not weak"
-		    if (!$Tangram::no_weakrefs and
-			!Scalar::Util::isweak($objects->{$oid}));
-	    my $x;
-	    warn __PACKAGE__."::unload_all: refcnt of oid $oid is $x"
-		if (!$Tangram::no_weakrefs and
-		    $x = Set::Object::rc($objects->{$oid}));
-	}
-    }
-    $self->{ids} = {};
-    $self->{objects} = {};
-    $self->{PREFETCH} = {};
-    $self->{scratch} = {};
-    print $Tangram::TRACE __PACKAGE__.": cache dumped\n"
-	if $Tangram::TRACE && ($Tangram::DEBUG_LEVEL > 0) ;
-
-    #$self->SUPER::unload_all();
-}
-
-# checks to see if an object ID ->isa the correct type, based on its
-# classtype
-sub oid_isa
-    {
-	my $self = shift;
-	my $oid = shift;
-	croak(pretty($oid)." is not an Object ID")
-	    unless defined ($oid) and $oid + 0 eq $oid;
-
-	my $class = shift;
-	my $classes = $self->{schema}->{classes};
-	carp "Class ".pretty($class)." is not defined in the schema",
-	    return undef
-		unless defined($class) and exists $classes->{$class};
-
-	my @bases = $self->{id2class}->{ ($self->split_id($oid))[1] + 0 };
-
-	my $seen = Set::Object->new();
-	while (my $base = shift @bases) {
-	    $seen->insert($classes->{$base}) or next;
-	    return 1 if $base eq $class;
-	    push @bases, @{ $classes->{$base}->{bases} }
-		if exists $classes->{$base}->{bases};
-	}
-
-	return undef;
-    }
-
 *reset = \&unload; # deprecated, use unload() instead
 
 sub DESTROY
 {
     my $self = shift;
-    $self->{db}->disconnect if $self->{db} && $self->{db_owned};
+    $self->{db}->disconnect if $self->{db};
 }
 
 package Tangram::Storage::Statement;
